function ctsp1

fprintf(1,'%c','Continous-Time Scaled Popov Example #1:')
fprintf(1,'%c \n',' ');
fprintf(1,'%c','16-th order Acoustic duct with uncertain damped natural frequencies')
fprintf(1,'%c \n',' ');
fprintf(1,'%c','Using 8th-order Centralized Dynamic Compensator')
fprintf(1,'%c \n',' ');
fprintf(1,'%c','From Erwin, Sparks, & Bernstein [1995]')
fprintf(1,'%c \n',' ');
fprintf(1,'%c \n',' ');

sigmavec = logspace(0,-8,50);

sigm = sqrt(sigmavec(50));

load aduct16norm
load uncertda

Dzu = Dzu*sigm;

dsofformat('RUNFILE',[8],[2,2],A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw,BdA,CeA,[],[],[],[])

clear
load RUNFILE

glist = [];
gammn = 1e-4;
gamlo = gammn;

% uncomment if restarting:
%
% load GAMMA
% [ngs,temp] = size(glist);
% gamup = glist(ngs,1);
% gamfl = (gamup + gamlo)/2;
% i = ngs+1;


% uncomment if starting from scratch:
%
	gamup = 1e6;
	gamfl = gamup;
	i = 1;

ctype = 3;
blk = [2,1;2,1];

gbnd1 = 1;

idata = [];

pstr = ['***************************************************'];
fprintf(1,'%c',pstr);
fprintf(1,'%c \n',' ');
pstr = ['Beginning gamma Iteration'];
fprintf(1,'%c',pstr);
fprintf(1,'%c \n',' ');
pstr = ['***************************************************'];
fprintf(1,'%c',pstr);
fprintf(1,'%c \n',' ');

while gbnd1 == 1

	gtest = 1;

	pstr = ['Iteration No. :',num2str(i)];
	fprintf(1,'%c' ,pstr);
	fprintf(1,'%c \n',' ');
	loadstr = ['load initNC8G',num2str(i-1)];

	pstr = ['Loading file initNC8G',num2str(i-1)];
	fprintf(1,'%c',pstr);
	fprintf(1,'%c \n',' ');

	eval(loadstr);

	K = daug(k1,k2,k3);

      	invLk = inv(eye(size(Dyu*K,1)) - Dyu*K);

      	Atilde = A + Bu*K*invLk*Cy;
      	Bwtilde = Bw + Bu*K*invLk*Dyw;
      	Cztilde = Cz + Dzu*K*invLk*Cy; 

      	Dzwtilde = Dzw + Dzu*K*invLk*Dyw;

     	if Dzwtilde ~= zeros(size(Dzwtilde))

		error('ERROR: Closed-Loop Feed-Through Matrx Dzwtilde is NONZERO!')

      	end

      	Bdtilde = Bd + Bu*K*invLk*Dyd;
      	Cetilde = Ce + Deu*K*invLk*Cy;

      	Vtilde  = Bwtilde*Bwtilde';
      	Rtilde  = Cztilde'*Cztilde;

	while gtest == 1

		pstr = ['Checking gamma = ',num2str(gamfl)];
		fprintf(1,'%c',pstr)
		fprintf(1,'%c \n',' ')		

		[WW,ZZ,fail] = pv_lmi(Atilde,Bdtilde,Cetilde,Rtilde,gamfl,blk);
		if fail == 0

			gamup = gamfl;
			gam = gamfl;
			gtest = 0;
			gamlo = gammn;

			pstr = ['Feasible gamma'];
			fprintf(1,'%c',pstr)
			fprintf(1,'%c \n',' ')

			pstr = ['Resetting Bounds: New Upper Bound gamma = ',num2str(gamup)];
			fprintf(1,'%c',pstr)
			fprintf(1,'%c \n',' ')
			pstr = ['                  New Lower Bound gamma = ',num2str(gamlo)];
			fprintf(1,'%c',pstr)
			fprintf(1,'%c \n',' ')
					


		else

			gamlo = gamfl;

			pstr = ['Infeasible gamma:    New Lower Bound gamma = ',num2str(gamlo)];
			fprintf(1,'%c',pstr)
			fprintf(1,'%c \n',' ')		

		end

		gamfl = (gamup + gamlo)/2;

		if (gamup - gamlo)/gamlo < .000001

			error('Error in Gamma Iteration: Cannot Lower Gamma To Previous Point')

		end

	end
	
	fprintf(1,'%c \n',' ')
	pstr = ['====================================================================='];
	fprintf(1,'%c',pstr)
	fprintf(1,'%c \n',' ')
	fprintf(1,'%c \n',' ')

	scale1 = abs(WW(1,1));
	scale2 = abs(ZZ(1,1));

	if scale1 < scale2
	
		WW = WW/scale1
		ZZ = ZZ/scale1

	else

		WW = WW/scale2
		ZZ = ZZ/scale2

	end

	pstr = ['Beginning Optimization with gamma = ',num2str(gam)];
	fprintf(1,'%c',pstr)
	fprintf(1,'%c \n',' ')
		
	save TEMP WW ZZ gam k1 k2 k3 
	load RUNFILE
	load TEMP

	savestr = ['save RUNFILE A Bu Bw Cy Cz Dyu Dyw Dzu Dzw Bd Ce Dyd Ded Dew Deu'];
	savestr = [savestr,' QL11 QL21 QL31 QR11 QR21 QR31 N k1 k2 k3 WW ZZ gam blk'];
	savestr = [savestr,' ctype kindex indexkc indexkr'];
	eval(savestr)
	clear savestr

	tic
	[fmin,info,noits] = optgain('RUNFILE','OUTFILE',10,0,5000,1,1);
	delt = toc;
	
	load OUTFILE;

	k1 = kopt1;k2 = kopt2;k3 =  kopt3;
	clear kopt1 kopt2 kopt3

	savestr = ['save initNC8G',num2str(i),' k1 k2 k3'];

	pstr = ['Saving Results to file initNC8G',num2str(i)];
	fprintf(1,'%c',pstr)
	fprintf(1,'%c \n',' ')

	eval(savestr);

	glist = [glist;gam,fmin,info,noits,delt];
	save GAMMA glist

	fprintf(1,'%c \n',' ')
	pstr = ['====================================================================='];
	fprintf(1,'%c',pstr)
	fprintf(1,'%c \n',' ')
	fprintf(1,'%c \n',' ')

	if i >= 2

		if abs(glist(i) - glist(i-1))/abs(glist(i-1)) < .01

			gbnd1 = 0;

			pstr = ['Last two values of gamma within 1% tolerance'];
			fprintf(1,'%c',pstr)
			fprintf(1,'%c \n',' ')
			pstr = ['Finishing gamma Iteration'];
			fprintf(1,'%c',pstr)
			fprintf(1,'%c \n',' ')

		end

	end

	i = i + 1;

end
		



		

		

		
