      subroutine fixpnfn(n,y,iflag,arcre,arcae,ansre,ansae,trace,a,nfe,
     &   arclen,yp,yold,ypold,qr,alpha,tz,pivot,w,wp,z0,z1,sspar,
     &   par,ipar)
c
c subroutine  fixpnf  finds a fixed point or zero of the
c n-dimensional vector function f(x), or tracks a zero curve
c of a general homotopy map rho(a,lambda,x).  for the fixed 
c point problem f(x) is assumed to be a c2 map of some ball 
c into itself.  the equation  x = f(x)  is solved by
c following the zero curve of the homotopy map
c
c  lambda*(x - f(x)) + (1 - lambda)*(x - a)  ,
c
c starting from lambda = 0, x = a.  the curve is parameterized
c by arc length s, and is followed by solving the ordinary
c differential equation  d(homotopy map)/ds = 0  for
c y(s) = (lambda(s), x(s)) using a hermite cubic predictor and a
c corrector which returns to the zero curve along the flow normal
c to the davidenko flow (which consists of the integral curves of
c d(homotopy map)/ds ).
c
c for the zero finding problem f(x) is assumed to be a c2 map
c such that for some r > 0,  x*f(x) >= 0  whenever norm(x) = r.
c the equation  f(x) = 0  is solved by following the zero curve
c of the homotopy map
c
c   lambda*f(x) + (1 - lambda)*(x - a)
c
c emanating from lambda = 0, x = a.
c
c  a  must be an interior point of the above mentioned balls.
c
c for the curve tracking problem rho(a,lambda,x) is assumed to
c be a c2 map from e**m x [0,1) x e**n into e**n, which for
c almost all parameter vectors a in some nonempty open subset
c of e**m satisfies
c
c  rank [d rho(a,lambda,x)/d lambda , d rho(a,lambda,x)/dx] = n
c
c for all points (lambda,x) such that rho(a,lambda,x)=0.  it is
c further assumed that
c
c           rank [ d rho(a,0,x0)/dx ] = n  .
c
c with a fixed, the zero curve of rho(a,lambda,x) emanating
c from  lambda = 0, x = x0  is tracked until  lambda = 1  by
c solving the ordinary differential equation
c d rho(a,lambda(s),x(s))/ds = 0  for  y(s) = (lambda(s), x(s)),
c where s is arc length along the zero curve.  also the homotopy
c map rho(a,lambda,x) is assumed to be constructed such that
c
c              d lambda(0)/ds > 0  .
c
c
c for the fixed point and zero finding problems, the user must supply 
c a subroutine  f(x,v)  which evaluates f(x) at x and returns the 
c vector f(x) in v, and a subroutine  fjac(x,v,k)  which returns in v 
c the kth column of the jacobian matrix of f(x) evaluated at x.  for 
c the curve tracking problem, the user must supply a subroutine  
c  rho(a,lambda,x,v,par,ipar)  which evaluates the homotopy map rho at 
c (a,lambda,x) and returns the vector rho(a,lambda,x) in v, and a
c subroutine  rhojac(a,lambda,x,v,k,par,ipar)  which returns in v the kth
c column of the n x (n+1) jacobian matrix [d rho/d lambda, d rho/dx]
c evaluated at (a,lambda,x).  fixpnf  directly or indirectly uses
c the subroutines  stepnf , tangnf , rootnf , root , f (or  rho ),
c fjac (or  rhojac ), d1mach , and the blas functions  ddot  and
c dnrm2 .  only  d1mach  contains machine dependent constants.
c no other modifications by the user are required.
c
c
c on input:
c
c n  is the dimension of x, f(x), and rho(a,lambda,x).
c
c y  is an arrray of length  n + 1.  (y(2),...,y(n+1)) = a  is the
c    starting point for the zero curve for the fixed point and 
c    zero finding problems.  (y(2),...,y(n+1)) = x0  for the curve
c    tracking problem.
c
c iflag  can be -2, -1, 0, 2, or 3.  iflag  should be 0 on the 
c    first call to  fixpnf  for the problem  x=f(x), -1 for the
c    problem  f(x)=0, and -2 for the problem  rho(a,lambda,x)=0.
c    in certain situations  iflag  is set to 2 or 3 by  fixpnf,
c    and  fixpnf  can be called again without changing  iflag.
c
c arcre , arcae  are the relative and absolute errors, respectively,
c    allowed the normal flow iteration along the zero curve.  if
c    arc?e .le. 0.0  on input it is reset to  .5*sqrt(ans?e) .
c    normally  arc?e should be considerably larger than  ans?e .
c
c ansre , ansae  are the relative and absolute error values used for
c    the answer at lambda = 1.  the accepted answer  y = (lambda, x)
c    satisfies
c
c       |y(1) - 1|  .le.  ansre + ansae           .and.
c
c       ||z||  .le.  ansre*||x|| + ansae          where
c
c    (.,z) is the newton step to y.
c
c trace  is an integer specifying the logical i/o unit for
c    intermediate output.  if  trace .gt. 0  the points computed on
c    the zero curve are written to i/o unit  trace .
c
c a(1:*)  contains the parameter vector  a .  for the fixed point
c    and zero finding problems, a  need not be initialized by the
c    user, and is assumed to have length  n.  for the curve
c    tracking problem, a  must be initialized by the user.
c
c yp(1:n+1)  is a work array containing the tangent vector to 
c    the zero curve at the current point  y .
c
c yold(1:n+1)  is a work array containing the previous point found
c    on the zero curve.
c
c ypold(1:n+1)  is a work array containing the tangent vector to 
c    the zero curve at  yold .
c
c qr(1:n,1:n+2), alpha(1:n), tz(1:n+1), pivot(1:n+1) , w(1:n+1) ,
c    wp(1:n+1) , z0(1:n+1) , z1(1:n+1)  are all work arrays used by
c    stepnf  to calculate the tangent vectors and newton steps.
c
c sspar(1:8) = (lideal, rideal, dideal, hmin, hmax, bmin, bmax, p)  is
c    a vector of parameters used for the optimal step size estimation.
c    if  sspar(j) .le. 0.0  on input, it is reset to a default value
c    by  fixpnf .  otherwise the input value of  sspar(j)  is used.
c    see the comments below and in  stepnf  for more information about
c    these constants.
c
c par(1:*) and ipar(1:*) are arrays for (optional) user parameters,
c    which are simply passed through to the user written subroutines
c    rho, rhojac.
c
c
c on output:
c
c n , trace , a  are unchanged.
c
c y(1) = lambda, (y(2),...,y(n+1)) = x, and y is an approximate
c    zero of the homotopy map.  normally lambda = 1 and x is a
c    fixed point(zero) of f(x).  in abnormal situations lambda
c    may only be near 1 and x is near a fixed point(zero).
c
c iflag =
c  -2   causes  fixpnf  to initialize everything for the problem
c       rho(a,lambda,x) = 0 (use on first call).
c
c  -1   causes  fixpnf  to initialize everything for the problem
c       f(x) = 0 (use on first call).
c
c   0   causes  fixpnf  to initialize everything for the problem
c       x = f(x) (use on first call).
c
c   1   normal return.
c
c   2   specified error tolerance cannot be met.  some or all of
c       arcre , arcae , ansre , ansae  have been increased to 
c       suitable values.  to continue, just call  fixpnf  again 
c       without changing any parameters.
c
c   3   stepnf  has been called 1000 times.  to continue, call
c       fixpnf  again without changing any parameters.
c
c   4   jacobian matrix does not have full rank.  the algorithm
c       has failed (the zero curve of the homotopy map cannot be
c       followed any further).
c
c   5   the tracking algorithm has lost the zero curve of the
c       homotopy map and is not making progress.  the error tolerances
c       arc?e  and  ans?e  were too lenient.  the problem should be
c       restarted by calling  fixpnf  with smaller error tolerances
c       and  iflag = 0 (-1, -2).
c
c   6   the normal flow newton iteration in  stepnf  or  rootnf
c       failed to converge.  the error tolerances  ans?e  may be too
c       stringent.
c
c   7   illegal input parameters, a fatal error.
c
c arcre , arcae , ansre , ansae  are unchanged after a normal return 
c    (iflag = 1).  they are increased to appropriate values on the 
c    return  iflag = 2 .
c
c nfe  is the number of function evaluations (= number of
c    jacobian evaluations).
c
c arclen  is the length of the path followed.
c
c
c
      double precision abserr,ansae,ansre,arcae,arclen,arcre,
     1   cursw,curtol,d1mach,dnrm2,h,hold,relerr,s
      integer iflag,iflagc,iter,jw,limit,limitd,n,nc,nfe,nfec,np1,
     1   trace
      logical crash,polsys,start
c
c ***** array declarations. *****
c
      double precision y(n+1),yp(n+1),yold(n+1),ypold(n+1),a(n),
     &   qr(n,n+2),alpha(n),tz(n+1),w(n+1),wp(n+1),z0(n+1),
     &   z1(n+1),sspar(8),par(1)
      integer pivot(n+1),ipar(1)
*-----------------------------------------------------------------------

      double precision pol(10,33),spol(10)
*-----------------------------------------------------------------------
c
c
c ***** end of dimensional information. *****
c
      save
c
c limitd  is an upper bound on the number of steps.  it may be
c changed by changing the following parameter statement:
      parameter (limitd=10000)
c
c switch from the tolerance  arc?e  to the (finer) tolerance  ans?e  if
c the curvature of any component of  y  exceeds  cursw.
      parameter (cursw=10.0)
c
c
c
c :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :
c set logical switch to reflect entry point.
      polsys=.false.
      go to 11
*      entry polynf(n,y,iflag,arcre,arcae,ansre,ansae,trace,a,nfe,
*     &   arclen,yp,yold,ypold,qr,alpha,tz,pivot,w,wp,z0,z1,sspar,
*     &   par,ipar)
*      polsys=.true.
11    continue
c
      if (n .le. 0  .or.  ansre .le. 0.0  .or.  ansae .lt. 0.0)
     &                                                     iflag=7
      if (iflag .ge. -2  .and.  iflag .le. 0) go to 20
      if (iflag .eq. 2) go to 120
      if (iflag .eq. 3) go to 90
c only valid input for  iflag  is -2, -1, 0, 2, 3.
      iflag=7
      return
c
c *****  initialization block.  *****
c
20    arclen=0.0
      if (arcre .le. 0.0) arcre=.5*sqrt(ansre)
      if (arcae .le. 0.0) arcae=.5*sqrt(ansae)
      nc=n
      nfec=0
      iflagc=iflag
      np1=n+1
c set initial conditions for first call to  stepnf .
      start=.true.
      crash=.false.
      hold=1.0
      h=.1
      s=0.0
      ypold(1)=1.0
      yp(1)=1.0
      y(1)=0.0
      do 40 jw=2,np1
        ypold(jw)=0.0
        yp(jw)=0.0
40    continue
c set optimal step size estimation parameters.
c let z[k] denote the newton iterates along the flow normal to the
c davidenko flow and y their limit.
c ideal contraction factor:  ||z[2] - z[1]|| / ||z[1] - z[0]||
      if (sspar(1) .le. 0.0) sspar(1)= .5
c ideal residual factor:  ||rho(a, z[1])|| / ||rho(a, z[0])||
      if (sspar(2) .le. 0.0) sspar(2)= .01
c ideal distance factor:  ||z[1] - y|| / ||z[0] - y||
      if (sspar(3) .le. 0.0) sspar(3)= .5
c minimum step size  hmin .
      if (sspar(4) .le. 0.0) sspar(4)= (sqrt(n+1.0)+4.0)*d1mach(4)
c maximum step size  hmax .
      if (sspar(5) .le. 0.0) sspar(5)= 0.2
c minimum step size reduction factor  bmin .
      if (sspar(6) .le. 0.0) sspar(6)= .1
c maximum step size expansion factor  bmax .
      if (sspar(7) .le. 0.0) sspar(7)= 3.0
c assumed operating order  p .
      if (sspar(8) .le. 0.0) sspar(8)= 2.0
c
c load  a  for the fixed point and zero finding problems.
      if (iflagc .ge. -1) then
        do 60 jw=2,np1
          a(jw-1)=y(jw)
60      continue
      endif
90    limit=limitd
c
c *****  end of initialization block.  *****
c
c
c *****  main loop.  *****
c
*----------------------------------------------
      kk=0 
*----------------------------------------------

120   do 400 iter=1,limit
      if (y(1) .lt. 0.0) then
        arclen=s
        iflag=5
        return
      endif
c
c set different error tolerance if the trajectory y(s) has any high 
c curvature components.
140   curtol=cursw*hold
      relerr=arcre
      abserr=arcae
      do 160 jw=1,np1
        if (abs(yp(jw)-ypold(jw)) .gt. curtol) then
          relerr=ansre
          abserr=ansae
          go to 200
        endif
160   continue
c
c take a step along the curve.
c     if(y(1).gt.0.90d0) sspar(5) = 0.005d0
200   call stepnf(nc,nfec,iflagc,start,crash,hold,h,relerr,abserr,
     +     s,y,yp,yold,ypold,a,qr,alpha,tz,pivot,w,wp,z0,z1,sspar,
     +     par,ipar)
c print latest point on curve if requested.
      if (trace .gt. 0) then
c        write (trace,217) iter,nfec,s,y(1),(y(jw),jw=2,np1)
c        write (6,217) iter,nfec,s,y(1),(y(jw),jw=2,np1)
c217     format(/' step',i5,3x,'nfe =',i5,3x,'arc length =',f9.4,3x,
c    &  'lambda =',f7.4,5x,'x vector:'/1p,(1x,6e12.4))
      endif
      nfe=nfec
*-----------------------------------------------------------------------
      write(6,221)iter,nfec,s,y(1)
c     write(2,221)iter,nfec,s,y(1)
 221   format('step=',i4,' nfe=',i5,' arc=',f9.4,' lambda=',f19.16)
      if(kk.ge.ipar(1)) then
       do 258 ige =1,ipar(1)-2
	 spol(ige)=spol(ige+2) 
	 do 258 jge=1,n+1
	 pol(ige,jge) = pol(ige+2,jge)
258    continue
       kk=ipar(1)-2
      endif
      kk=kk+2 
	 spol(kk) = s
	 spol(kk-1)= s 
      do 257 jge=1,n+1
	 pol(kk-1,jge) = y(jge)
	 pol(kk,jge) = yp(jge)
257   continue
*-----------------------------------------------------------------------
c check if the step was successful.
      if (iflagc .gt. 0) then
        arclen=s
        iflag=iflagc
        return
      endif
      if (crash) then
c return code for error tolerance too small.
        iflag=2
c change error tolerances.
        if (arcre .lt. relerr) arcre=relerr
        if (ansre .lt. relerr) ansre=relerr
        if (arcae .lt. abserr) arcae=abserr
        if (ansae .lt. abserr) ansae=abserr
c change limit on number of iterations.
        limit=limit-iter
        return
      endif
c
      if (y(1) .ge. 1.0) then
c
c use hermite cubic interpolation and newton iteration to get the 
c answer at lambda = 1.0 .
c
c save  yold  for arc length calculation later.
        do 260 jw=1,np1
          z0(jw)=yold(jw)
260     continue
        call rootg(nc,iflagc,ansre,ansae,y,pol,spol,w,par,ipar)
c
        nfe=nfec
        iflag=1
c set error flag if  rootnf  could not get the point on the zero
c curve at  lambda = 1.0  .
        if (iflagc .gt. 0) iflag=iflagc
c calculate final arc length.
        do 290 jw=1,np1
          w(jw)=y(jw) - z0(jw)
290     continue
        arclen=s - hold + dnrm2(np1,w,1)
        return
      endif
c
400   continue
c
c *****  end of main loop.  *****
c
c lambda has not reached 1 in 1000 steps.
      iflag=3
      arclen=s
      return
c
      end
