
c
c    this subroutine was taken from the book:
c      elementary numerical analysis, conte & deboor, prentice-hall, 1972
c      input
c
c          np1    -the number of interpolating points.
c
c          y       -an array of size np1 containing the interpolating points
c                  it is assumed that:
c                  1) -all repeated interpolating points appear together
c
c                  2) -if y(i)=y(i+k) and if y(i-1) .ne. y(i),
c                      or if i=1 then
c                     f(i+j)= value of the j-th derivative of f(x)
c                      at x=y(i) for j=0,1.....k.
c                     
c           f      -an array of size np1 containing the valuew f(y(i)),
c                       which correspond to the interpolating points
c                     y(i), i=1,...,np1.
c
c          npoint -the number of equally spaced points at which the
c                    interpolating polynomial is to be evaluated.
c                    
c           x      -the initial point at which the interpolating
c                    polynomial will be evaluated.
c
c          dx     -the value of the spacing factor. x will be
c                   incremented by dx and the interpolating polynomial
c                  will be evaluated at npoint of these points
c
c
c     output
c
c           f      -contains the divided difference coefficients of the
c                   newton interpolating polynomial.
c
c
c          result -an array of length npoint containing the values
c                  of interpolating polynomial at the equally spaced
c                  points specified.
c
c          np1,y,npoint,dx are unchanged on return.
c
           subroutine interp(np1,y,f,npoint,x, dx,result)
      implicit real*8 (a-h,o-z)
      dimension y(np1),f(np1),result(npoint)
      n = np1-1
      if (n .eq. 0) go to 20
        do 10 k = 1,n
          floatk = k
          np1mk = np1-k
          flast = f(1)
          do 9 i = 1,np1mk
            ipk = i+k
            dy =y(ipk)-y(i)
            if (dy .eq. 0.0d0) go to 8
              f(i) = (f(i+1)-flast)/dy
              flast = f(i+1)
            go to 9
   8        f(i) = f(i+1)/floatk
   9      continue
        f(np1mk+1) = flast
  10   continue
  20  do 30 j = 1,npoint
         pnofx = 0.0d0
         do 29 i = 1,np1
  29       pnofx = f(i)+(x-y(i))*pnofx
         result(j)=pnofx
         x = x+dx
  30  continue
      return
      end
