
function [ Ac,Bc,Cc,Dc ] = balcred( Ac0,Bc0,Cc0,Dc0, Phat,Qhat, Nc )
%
%BALCRED Given Ac0, Bc0 and Cc0 (gain matrices of a full order controller),
%       and Phat, Qhat (corresponding grammians), this code finds a
%       reduced order controller { Ac, Bc, Cc, Dc } of order Nc using
%       balanced controller reduction.  The controller gains and
%       grammians can be either discrete or continuous time.
%
%       Calling order is:
%
%       [ Ac,Bc,Cc,Dc ] = balcred( Ac0,Bc0,Cc0,Dc0, Phat,Qhat, Nc ) 
%
%       Functions: CHOL and SVD from matlab.
%

%       910829  A. Daubendiek   created
%       910903  A. Daubendiek   Transposed value returned by chol.
%                               Corrected coding error in step 3.
%                               Added step 5 (\ instead of *).
%
%       step 1
%
LPhat = chol( Phat )' ;
LQhat = chol( Qhat )' ;
%
%       step 2
%
[ U, omega, V ] = svd( LPhat'*LQhat ) ;
%
%       step 3
%
so = sqrt( omega ) ;
T = LQhat*V/so ;
invT = so\U'*LPhat' ;
%
%       step 4
%
Ac = invT*Ac0*T ;
Bc = invT*Bc0 ;
Cc = Cc0*T ;
Dc = Dc0 ;
%
%       step 4 and 5
%
Ac = Ac(1:Nc,1:Nc) ;
Bc = Bc(1:Nc,:) ;
Cc = Cc(:,1:Nc) ;
%
clear LPhat LQhat U omega V so T invT
%
%       return
%

