
function [Acf,Bcf,Ccf,cost] = rlqgly(A,B,C,D,nc,gamma0,gamma,beta,E1,E2,E1i,E2i,D1,D2)
% Find the sizes of the system and check the dimensions 
[nr,n] = size(A);
if (nr ~= n), error('Nonsquare A matrix'); end;

[nb,m] = size(B);
if (n~=nb), error('Incorrectly dimensioned B matrix'); end;

[l,nr] = size(C);
if (nr~=n), error('Incorrectly dimensioned C matrix'); end;

% Set the starting gamma
if (gamma <= 0.0), error('gamma must be greater than zero'); end;

R1 =  E1'* E1;
R2 =  E2'* E2;
R1i = E1i' * E1i;
R2i = E2i' * E2i;
V1 = D1* D1';
V2 = D2* D2';

%*********************************************************
% Transform the orginal system to the required form.
% Check whether the 12 block of A is zero.  Do a Schur
% decomposition if not.

 U = eye(n,n);
 for i = 1:nc
   for j = (nc+1):n
     if A(i, j) ~= 0.0, 
       [U,T] = schur(A'); 
       break;
     end 
    end
 end

% Transform all the other matrices
  A = T';
  B = U * B;
  C = C * U';
  E1 = E1*U';
  E1i = E1i *U';
  D1 = U*D1;
  R1 = U * R1 * U';
  R1i = U * R1i * U';
  V1 = U * V1 * U';
  
%**********************************************************
% Set all the initial matrices

A0 = A;
 for i = 1:(n/2),
   A22 = A0((2*i-1):(2*i),(2*i-1):(2*i));
   y = eig(A22);
   if y(1) >= 0 | y(2) >= 0,  
   A0((2*i-1):(2*i),(2*i-1):(2*i)) = A22 -(max(abs(y(1)), abs(y(2)))+1)*eye(2); 
   end
 end

%A0 = A0 - 10.0*eye(n);

B0 = B;
C0 = C;
R20 = R2;
R2i0 = R2i;
V20 = beta * V2;
V10 = V1;
R10 = zeros(n,n);
%R10(1:nc,1:nc) = R1(1:nc,1:nc) + 0.01*eye(nc); 
R10(1:nc,1:nc) = R1(1:nc,1:nc); 
R1i0 = R1i;

%***********************************************************
% Find the initial reduced order compensator

  Sigma0 = B * inv(R20) * B';
  bSigma0 = C' * inv(V20) * C;
% Solve for P
  P = are(A0, Sigma0, R10);
% Solve for Q
  Q = are(A0', bSigma0, V10);

% (Ac, Bc, Cc)
  Cc = (-1)*inv(R20)* B'* P;
  Bc = Q * C' * inv(V20); 
  Ac = A0 + B*Cc - Bc * C;
  Dc = 0;

% Solve for P^ and Q^
  
  As = A0 - Q * bSigma0;
  Pp = P* Sigma0 * P;

  Ph = lyap(As',Pp);

  As = A0 - Sigma0 * P;
  Qp = Q* bSigma0 * Q;

  Qh = lyap(As,Qp);

[Ac0, Bc0, Cc0, Dc0] = balcred(Ac,Bc,Cc,Dc,Ph,Qh,nc);

%**********************************************************
% Initialization is done.  Write all the necessary matrices
% to a datafile.

arcre = 1.0e-5;
ansre = 1.0e-9;

in = fopen('in.dat','w');
fprintf(in, '%u  %u  %u  %u\n',n,nc,m,l);
fprintf(in, '%e  %e  %e  %e\n',arcre,ansre,gamma0,gamma);

for i=1:n
  for j=1:n
    fprintf(in, '%17.9e\n',A(i,j));
  end
end

for i=1:n
  for j=1:n
    fprintf(in, '%17.9e\n',A0(i,j));
  end
end

for i=1:n
  for j=1:m
    fprintf(in, '%17.9e\n',B(i,j));
  end
end

for i=1:l
  for j=1:n
    fprintf(in, '%17.9e\n',C(i,j));
  end
end

for i=1:nc
  for j=1:nc
    fprintf(in, '%17.9e\n',Ac0(i,j));
  end
end

for i=1:nc
  for j=1:l
    fprintf(in, '%17.9e\n',Bc0(i,j));
  end
end

for i=1:m
  for j=1:nc
    fprintf(in, '%17.9e\n',Cc0(i,j));
  end
end

for i=1:n
  for j=1:n
    fprintf(in, '%17.9e\n',R10(i,j));
  end
end

for i=1:n
  for j=1:n
    fprintf(in, '%17.9e\n',R1(i,j));
  end
end

for i=1:m
  for j=1:m
    fprintf(in, '%17.9e\n',R20(i,j));
  end
end

for i=1:m
  for j=1:m
    fprintf(in, '%17.9e\n',R2(i,j));
  end
end

for i=1:n
  for j=1:n
    fprintf(in, '%17.9e\n',R1i(i,j));
  end
end

for i=1:m
  for j=1:m
    fprintf(in, '%17.9e\n',R2i(i,j));
  end
end

for i=1:n
  for j=1:n
    fprintf(in, '%17.9e\n',V10(i,j));
  end
end

for i=1:n
  for j=1:n
    fprintf(in, '%17.9e\n',V1(i,j));
  end
end

for i=1:l
  for j=1:l
    fprintf(in, '%17.9e\n',V20(i,j));
  end
end

for i=1:l
  for j=1:l
    fprintf(in, '%17.9e\n',V2(i,j));
  end
end
fclose(in);

%************************************************************
% Run the program

!reLQGly

%*************************************************************
% Final processing: find the H_\infty norm.

load hi.dat;

iflag = hi(1,1);
nc = hi(2,1);
gamma=hi(3,1);

if(iflag == 2), error('Specified error tolerrance can not be met.'); end;

if(iflag == 3), error('Stepnf  has been called 1000 times.'); end;

if(iflag == 4), error('jacobian matrix does not have full rank'); end;

if(iflag == 5), error('Tracking algorithm has lost the zero curve.'); 
fprintf(1,'Use smaller arcre and ansre in h2hiinf.m\n'); end;

if(iflag == 6), 
error('Make gamma0 smaller or ancre and ansre in h2hiinf.m larger and try again.'); end;

k=3; 

for i=1:nc
 for j=1:nc
 k=k+1;
 Acf(i,j) = hi(k,1);
 end
end

 for j=1:nc
 k=k+1;
 bct(1,j) = hi(k,1);
 end

Bcf=bct' ;

 for j=1:nc
 k=k+1;
 Ccf(1:1,j) = hi(k,1);
 end

 k=k+1;
 scost = hi(k,1);
 k=k+1;
 cost  = hi(k,1);
