      subroutine initp3(ld,n,nm,m,l,nn,c1,c2,d,bvbt,ctrc,b,c,v,r,y,hom,
     &                  q,p,tau,f10,f20,f30,ww,wi,sigma,temp1,temp2,
     &                  temp3,qr,wa,fvec,s1)
*
* Strategy 3 for choosing the starting point for homotopy
*
* Input parameters:
* ld - leading dimension of matrices
* n,nm,m,l - dimensions of system and model
* nn - Jacobian size
* c1,c2 - parameters used in definition of D
* d,bvbt,ctrc,b,c,v,r - D, B V B^t, C^t R C, B, C, V, R
*
* Output parameters:
* y - point in vector form
* hom - parameter indicating which homotopy map will be used, depends
*       on the result of quasi-Newton iteration
*
* Working storage:
* q,p,tau,f10,f20,f30,ww,wi,sigma,temp1,temp2,temp3,qr - working matrices
* wa,fvec,s1 - working arrays
*
      integer hom,i,l,ld,m,n,nm,nn,z
      double precision b(ld,ld),bvbt(ld,ld),c(ld,ld),c1,c2,ctrc(ld,ld)
      double precision d(ld,ld),f10(ld,ld),f20(ld,ld),f30(ld,ld)
      double precision fvec(nn),p(ld,ld),q(ld,ld),qr(nn,nn),r(ld,ld)
      double precision s1(ld),sigma(ld,ld),small,t1,t2,tau(ld,ld)
      double precision temp1(ld,ld),temp2(ld,ld),temp3(ld,ld),v(ld,ld)
      double precision wa(nn*(nn+13)/2),wi(ld,ld),ww(ld,ld),y(nn)
*
      z = ld * ld
      small = 1.d-3
*
* Augment B to TEMP1
*
      call setmat(ld,0.d0,temp1,n,nm)
      call scmat(ld,1.d0,b,temp1,n,m)
      t1 = 1.d-1
      t2 = 1.d-1 / dble(n*n)
      do 311 i = 1,nm
        if (dabs(temp1(i,i)) .lt. t1) temp1(i,i) = t1
        t1 = t1 + t2
311   continue
*
* Compute augmented (rank-nm) product B V B^t
*
      call setmat(ld,0.d0,temp2,nm,nm)
      call scmat(ld,1.d0,v,temp2,m,m)
      if (nm .gt. m) then
        do 301 i = m+1,nm
          temp2(i,i) = 1.d0
301     continue
      endif
      call mulmat(ld,temp1,temp2,temp3,n,nm,nm,1)
      call mulmat(ld,temp3,temp1,temp2,n,n,nm,3)
      t1 = 1.d0 / (2.d0 * c1)
      call scmat(ld,t1,temp2,q,n,n)
*
* Augment C to TEMP1
*
      call setmat(ld,0.d0,temp1,nm,n)
      call scmat(ld,1.d0,c,temp1,l,n)
      t1 = 1.d-1
      t2 = 1.d-1 / dble(n*n)
      do 313 i = 1,nm
        if (dabs(temp1(i,i)) .lt. t1) temp1(i,i) = t1
        t1 = t1 + t2
313   continue
*
* Compute augmented (rank-nm) product C^t R C
*
      call setmat(ld,0.d0,temp2,nm,nm)
      call scmat(ld,1.d0,r,temp2,l,l)
      if (nm .gt. l) then
        do 302 i = l+1,nm
          temp2(i,i) = 1.d0
302     continue
      endif
      call mulmat(ld,temp1,temp2,temp3,n,nm,nm,2)
      call mulmat(ld,temp3,temp1,temp2,n,n,nm,1)
      t1 = 1.d0 / (2.d0 * c1)
      call scmat(ld,t1,temp2,p,n,n)
*
* Compute (W,U,Sigma) for given (Q,P)
*
      call cg(ld,n,nm,q,p,ww,wi,sigma,tau,temp1,temp2,temp3,
     &        wa(1),wa(z+1),wa(2*z+1),wa(3*z+1),wa(4*z+1),wa(5*z+1),
     &        wa(6*z+1),wa(7*z+1),wa(8*z+1),wa(9*z+1),wa(10*z+1),
     &        wa(11*z+1))
*
c     write(2,231)
c231   format(/'sigma')
c     call writem(ld,sigma,nm,nm)
*
* Quasi-Newton iteration
*
*      call apprd(ld,n,nm,nn,d,bvbt,ctrc,y,hom,ww,wi,sigma,f10,f20,f30,
*     &           temp1,temp2,temp3,qr,wa,fvec,s1)
*      if (hom .ne. 1) hom = 0
*
      return
      end
