*
*     RHO
*
      subroutine rho(aa,lambda,x,v,p,ip)
*
* Subroutine RHO is the interface between HOMPACK and RRHO. Its purpose
* is to process parameters passed thru arrays P and IP, and send it to
* RRHO.
*
      integer ld,n,nm,type,hom,z
      integer ip(*)
      double precision aa(*),x(*),v(*),p(*)
      double precision lambda
*
      ld = ip(1)
      n = ip(2)
      nm = ip(3)
      type = ip(4)
      hom = ip(5)
      z = ld * ld
*
      call rrho(ld,n,nm,type,hom,lambda,x,v,p(1),p(z+1),p(2*z+1),
     &          p(3*z+1),p(4*z+1),p(5*z+1),p(6*z+1),p(7*z+1),p(8*z+1),
     &          p(9*z+1),p(10*z+1),p(11*z+1),p(12*z+1),p(13*z+1),
     &          p(14*z+1))
*
      return
      end
*
*-----------------------------------------------------------------------
*-----------------------------------------------------------------------
*
*     RRHO
*
      subroutine rrho(ld,n,nm,type,hom,lambda,x,v,a,d,bvbt,ctrc,f10,
     &                f20,f30,ww,wi,sigma,temp1,temp2,temp3,aal,mat)
*
* RRHO evaluates rho(a,lambda,x) at given point.
*
* Input parameters:
* ld - leading dimension of matrices
* n,nm - dimensions of system and model
* type - indicates whether RRHO is used for curve tracking (type = 1)
*        or quasi-Newton iterations (type = 2 or 3)
* hom - indicates type of homotopy map: hom = 0 indicates Newton
*       homotopy, hom = 1 indicates rho(a,lambda,x) = F(a,lambda,x)
* lambda,x - input point (lambda,x)
* a,d,bvbt,ctrc - A,D, B V B^t, C^t R C
* f10,f20,f30 - F(0,X(0))
* ww,wi,sigma,temp1,temp2,temp3,aal,mat - work matrices
*
* Output parameters:
* v - evaluated function
*
      integer ld,n,nm,type,hom,xtyp
      double precision v(*)
      double precision a(ld,ld),d(ld,ld),bvbt(ld,ld),ctrc(ld,ld)
      double precision f10(ld,ld),f20(ld,ld),f30(ld,ld)
      double precision ww(ld,ld),wi(ld,ld),sigma(ld,ld)
      double precision temp1(ld,ld),temp2(ld,ld),temp3(ld,ld)
      double precision aal(ld,ld),mat(ld,ld)
      double precision lambda,t1
*
* Transform point from array to matrix form (W,U,Sigma)
*
      xtyp = 1
      call xwus(ld,n,nm,xtyp,ww,wi,sigma,x)
*
* Compute AAL = lambda * A + (1-lambda) * D
*
      t1 = 1.d0 - lambda
      call scmat(ld,lambda,a,temp1,n,n)
      call scmat(ld,t1,d,temp2,n,n)
      call addmat(ld,temp1,temp2,aal,n,n,1)
*
* Determine which matrix is used as A(lambda)
*
      if (type .eq. 1) call scmat(ld,1.d0,aal,mat,n,n)
      if (type .eq. 2) call scmat(ld,1.d0,d,mat,n,n)
      if (type .eq. 3) call scmat(ld,1.d0,a,mat,n,n)
*
* Compute F1(a,lambda,x)
*
      call mulmat(ld,wi,mat,temp1,nm,n,n,1)
      call mulmat(ld,temp1,ww,temp2,nm,nm,n,1)
      call mulmat(ld,temp2,sigma,temp1,nm,nm,nm,1)
      call mulmat(ld,temp1,ww,temp3,nm,n,nm,3)
*
      call mulmat(ld,sigma,ww,temp1,nm,n,nm,3)
      call mulmat(ld,temp1,mat,temp2,nm,n,n,3)
      call addmat(ld,temp2,temp3,temp3,nm,n,1)
*
      call mulmat(ld,wi,bvbt,temp1,nm,n,n,1)
      call addmat(ld,temp1,temp3,temp3,nm,n,1)
*
      if (type .eq. 1 .and. hom .ne. 1) then
        t1 = 1.d0 - lambda
        call scmat(ld,t1,f10,temp1,nm,n)
        call addmat(ld,temp1,temp3,temp3,nm,n,2)
      endif
*
* Transform result to vector form
*
      xtyp = 2
      call xmat(ld,xtyp,nm,n,temp3,v)
*
* Compute F2(a,lambda,x)
*
      call mulmat(ld,wi,sigma,temp1,n,nm,nm,2)
      call mulmat(ld,temp1,wi,temp2,n,n,nm,1)
      call mulmat(ld,temp2,mat,temp1,n,n,n,1)
      call mulmat(ld,temp1,ww,temp3,n,nm,n,1)
*
      call mulmat(ld,mat,wi,temp1,n,nm,n,4)
      call mulmat(ld,temp1,sigma,temp2,n,nm,nm,1)
      call addmat(ld,temp2,temp3,temp3,n,nm,1)
*
      call mulmat(ld,ctrc,ww,temp1,n,nm,n,1)
      call addmat(ld,temp1,temp3,temp3,n,nm,1)
*
      if (type .eq. 1 .and. hom .ne. 1) then
        call scmat(ld,t1,f20,temp1,n,nm)
        call addmat(ld,temp1,temp3,temp3,n,nm,2)
      endif
*
* Transform result to vector form
*
      xtyp = 2
      call xmat(ld,xtyp,n,nm,temp3,v(1+n*nm))
*
* Compute F3(a,lambda,x)
*
      call mulmat(ld,wi,ww,temp3,nm,nm,n,1)
      do 351 i = 1,nm
        temp3(i,i) = temp3(i,i) - 1.d0
351   continue
*
      if (type .eq. 1 .and. hom .ne. 1) then
        call scmat(ld,t1,f30,temp1,nm,nm)
        call addmat(ld,temp1,temp3,temp3,nm,nm,2)
      endif
*
* Transform result to vector form
*
      xtyp = 2
      call xmat(ld,xtyp,nm,nm,temp3,v(1+2*n*nm))
*
      return
      end
*
*-----------------------------------------------------------------------
*-----------------------------------------------------------------------
*
*     RHOJAC
*
      subroutine rhojac(aa,lambda,x,v,kk,p,ip)
*
* Subroutine RHOJAC is the interface between HOMPACK and RRHOJ. Its purpose
* is to process parameters passed thru arrays P and IP, and send it to
* RRHOJ.
*
      integer ld,n,nm,type,hom,kk,z
      integer ip(*)
      double precision aa(*),x(*),v(*),p(*)
      double precision lambda
*
      ld = ip(1)
      n = ip(2)
      nm = ip(3)
      type = ip(4)
      hom = ip(5)
      z = ld * ld
*
      call rrhoj(ld,n,nm,type,hom,lambda,x,v,kk,p(1),p(z+1),
     &          p(2*z+1),p(3*z+1),p(4*z+1),p(5*z+1),p(6*z+1),p(7*z+1),
     &          p(8*z+1),p(9*z+1),p(10*z+1),p(11*z+1),p(12*z+1),
     &          p(13*z+1),p(14*z+1),p(15*z+1))
*
      return
      end
*
*-----------------------------------------------------------------------
*-----------------------------------------------------------------------
*
*     RRHOJ
*
      subroutine rrhoj(ld,n,nm,type,hom,lambda,x,v,kk,a,d,bvbt,ctrc,
     &                 f10,f20,f30,ww,wi,sigma,temp1,temp2,temp3,aal,
     &                 mat,amd)
*
* RRHOJ evaluates kk-th column of Jacobian at given point.
*
* Input parameters:
* ld - leading dimension of matrices
* n,nm - dimensions of system and model
* type - indicates whether RRHO is used for curve tracking (type = 1)
*        or quasi-Newton iterations (type = 2 or 3)
* hom - indicates type of homotopy map: hom = 0 indicates Newton
*       homotopy, hom = 1 indicates rho(a,lambda,x) = F(a,lambda,x)
* lambda,x - input point (lambda,x)
* kk - column number
* a,d,bvbt,ctrc - A,D, B V B^t, C^t R C
* f10,f20,f30 - F(0,X(0))
* ww,wi,sigma,temp1,temp2,temp3,aal,mat - work matrices
*
* Output parameters:
* v - evaluated function
*
      integer ld,n,nm,type,hom,i,j,k,kk,der,xtyp
      double precision v(*)
      double precision a(ld,ld),d(ld,ld),bvbt(ld,ld),ctrc(ld,ld)
      double precision f10(ld,ld),f20(ld,ld),f30(ld,ld)
      double precision ww(ld,ld),wi(ld,ld),sigma(ld,ld)
      double precision temp1(ld,ld),temp2(ld,ld),temp3(ld,ld)
      double precision aal(ld,ld),mat(ld,ld),amd(20,20)
      double precision lambda,t1
*
* Transform point from array to matrix form (W,U,Sigma)
*
      xtyp = 1
      call xwus(ld,n,nm,xtyp,ww,wi,sigma,x)
*
* Compute AAL = lambda * A + (1-lambda) * D
*
      t1 = 1.d0 - lambda
      call scmat(ld,lambda,a,temp1,n,n)
      call scmat(ld,t1,d,temp2,n,n)
      call addmat(ld,temp1,temp2,aal,n,n,1)
*
* Determine which matrix is used as A(lambda)
*
      if (type .eq. 1) call scmat(ld,1.d0,aal,mat,n,n)
      if (type .eq. 2) call scmat(ld,1.d0,d,mat,n,n)
      if (type .eq. 3) call scmat(ld,1.d0,a,mat,n,n)
*
* Determine variable with respect to which the derivative is sought
* der=4 - lambda, der=1 - W, der=2 - U, der=3 - Sigma
*
      if (type .eq. 1) k = kk
      if (type .eq. 2) k = kk + 1
      if (type .eq. 3) k = kk + 1
*
      if (k .eq. 1) der = 4
      if (k .gt. 1 .and. k .le. n*nm+1) der = 1
      if (k .gt. n*nm+1 .and. k .le. 2*n*nm+1) der = 2
      if (k .gt. 2*n*nm+1) der = 3
*
* Determine position (i,j) in W, U or Sigma
*
      if (der .eq. 1) call compij(k-1,n,nm,i,j)
      if (der .eq. 2) call compij(k-n*nm-1,nm,n,i,j)
      if (der .eq. 3) call compij(k-2*n*nm-1,nm,nm,i,j)
*.......................................................................
*
* Derivatives with respect to Wij
*
      if (der .eq. 1) then
*
* Compute derivatives of F1(a,lambda,x)
*
        call mulmat(ld,wi,mat,temp1,nm,n,n,1)
        call mulmat(ld,temp1,ww,temp2,nm,nm,n,1)
        call mulmat(ld,temp2,sigma,temp1,nm,nm,nm,1)
        call setmat(ld,0.d0,temp3,nm,n)
        call rowcol(ld,temp3,temp1,nm,i,j,4)
*
        call mulmat(ld,wi,mat,temp1,nm,n,n,1)
        call setmat(ld,0.d0,temp2,nm,nm)
        call rowcol(ld,temp2,temp1,nm,j,i,4)
        call mulmat(ld,temp2,sigma,temp1,nm,nm,nm,1)
        call mulmat(ld,temp1,ww,temp2,nm,n,nm,3)
        call addmat(ld,temp2,temp3,temp3,nm,n,1)
*
        call setmat(ld,0.d0,temp1,nm,n)
        call rowcol(ld,temp1,sigma,nm,i,j,4)
        call mulmat(ld,temp1,mat,temp2,nm,n,n,3)
        call addmat(ld,temp2,temp3,temp3,nm,n,1)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,nm,n,temp3,v)
*
* Compute derivatives of F2(a,lambda,x)
*
        call mulmat(ld,wi,sigma,temp1,n,nm,nm,2)
        call mulmat(ld,temp1,wi,temp2,n,n,nm,1)
        call mulmat(ld,temp2,mat,temp1,n,n,n,1)
        call setmat(ld,0.d0,temp3,n,nm)
        call rowcol(ld,temp3,temp1,n,j,i,4)
        call rowcol(ld,temp3,ctrc,n,j,i,4)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,n,nm,temp3,v(1+n*nm))
*
* Compute derivatives of F3(a,lambda,x)
*
        call setmat(ld,0.d0,temp3,nm,nm)
        call rowcol(ld,temp3,wi,nm,j,i,4)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,nm,nm,temp3,v(1+2*n*nm))
      endif
*.......................................................................
*
* Derivatives with respect to Uij
*
      if (der .eq. 2) then
*
* Compute derivatives of F1(a,lambda,x)
*
        call mulmat(ld,mat,ww,temp1,n,nm,n,1)
        call mulmat(ld,temp1,sigma,temp2,n,nm,nm,1)
        call mulmat(ld,temp2,ww,temp1,n,n,nm,3)
        call setmat(ld,0.d0,temp3,nm,n)
        call rowcol(ld,temp3,temp1,n,i,j,1)
        call rowcol(ld,temp3,bvbt,n,i,j,1)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,nm,n,temp3,v)
*
* Compute derivatives of F2(a,lambda,x)
*
        call mulmat(ld,sigma,wi,temp1,nm,n,nm,1)
        call mulmat(ld,temp1,mat,temp2,nm,n,n,1)
        call mulmat(ld,temp2,ww,temp1,nm,nm,n,1)
        call setmat(ld,0.d0,temp3,n,nm)
        call rowcol(ld,temp3,temp1,nm,j,i,1)
*
        call mulmat(ld,wi,sigma,temp1,n,nm,nm,2)
        call setmat(ld,0.d0,temp2,n,n)
        call rowcol(ld,temp2,temp1,n,j,i,4)
        call mulmat(ld,temp2,mat,temp1,n,n,n,1)
        call mulmat(ld,temp1,ww,temp2,n,nm,n,1)
        call addmat(ld,temp2,temp3,temp3,n,nm,1)
*
        call setmat(ld,0.d0,temp1,n,nm)
        call rowcol(ld,temp1,mat,n,i,j,3)
        call mulmat(ld,temp1,sigma,temp2,n,nm,nm,1)
        call addmat(ld,temp2,temp3,temp3,n,nm,1)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,n,nm,temp3,v(1+n*nm))
*
* Compute derivatives of F3(a,lambda,x)
*
        call setmat(ld,0.d0,temp3,nm,nm)
        call rowcol(ld,temp3,ww,nm,i,j,1)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,nm,nm,temp3,v(1+2*n*nm))
      endif
*.......................................................................
*
* Derivatives with respect to SIGMAij
*
      if (der .eq. 3) then
*
* Compute derivatives of F1(a,lambda,x)
*
        call mulmat(ld,wi,mat,temp1,nm,n,n,1)
        call mulmat(ld,temp1,ww,temp2,nm,nm,n,1)
        call setmat(ld,0.d0,temp1,nm,nm)
        call rowcol(ld,temp1,temp2,nm,j,i,4)
        call mulmat(ld,temp1,ww,temp3,nm,n,nm,3)
*
        call mulmat(ld,ww,mat,temp1,nm,n,n,4)
        call setmat(ld,0.d0,temp2,nm,n)
        call rowcol(ld,temp2,temp1,n,i,j,1)
        call addmat(ld,temp2,temp3,temp3,nm,n,1)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,nm,n,temp3,v)
*
* Compute derivatives of F2(a,lambda,x)
*
        call setmat(ld,0.d0,temp1,n,nm)
        call rowcol(ld,temp1,wi,n,j,i,3)
        call mulmat(ld,temp1,wi,temp2,n,n,nm,1)
        call mulmat(ld,temp2,mat,temp1,n,n,n,1)
        call mulmat(ld,temp1,ww,temp3,n,nm,n,1)
*
        call mulmat(ld,mat,wi,temp1,n,nm,n,4)
        call setmat(ld,0.d0,temp2,n,nm)
        call rowcol(ld,temp2,temp1,n,j,i,4)
        call addmat(ld,temp2,temp3,temp3,n,nm,1)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,n,nm,temp3,v(1+n*nm))
*
* Compute derivatives of F3(a,lambda,x)
*
        call setmat(ld,0.d0,temp3,nm,nm)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,nm,nm,temp3,v(1+2*n*nm))
      endif
*.......................................................................
*
* Derivatives with respect to lambda
*
      if (der .eq. 4) then
*
        call addmat(ld,a,d,amd,n,n,3)
*
* Compute derivatives of F1(a,lambda,x)
*
        call mulmat(ld,wi,amd,temp1,nm,n,n,1)
        call mulmat(ld,temp1,ww,temp2,nm,nm,n,1)
        call mulmat(ld,temp2,sigma,temp1,nm,nm,nm,1)
        call mulmat(ld,temp1,ww,temp3,nm,n,nm,3)
*
        call mulmat(ld,sigma,ww,temp1,nm,n,nm,3)
        call mulmat(ld,temp1,amd,temp2,nm,n,n,3)
        call addmat(ld,temp2,temp3,temp3,nm,n,1)
*
        if (hom .ne. 1) call addmat(ld,f10,temp3,temp3,nm,n,1)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,nm,n,temp3,v)
*
* Compute derivatives of F2(a,lambda,x)
*
        call mulmat(ld,wi,sigma,temp1,n,nm,nm,2)
        call mulmat(ld,temp1,wi,temp2,n,n,nm,1)
        call mulmat(ld,temp2,amd,temp1,n,n,n,1)
        call mulmat(ld,temp1,ww,temp3,n,nm,n,1)
*
        call mulmat(ld,amd,wi,temp1,n,nm,n,4)
        call mulmat(ld,temp1,sigma,temp2,n,nm,nm,1)
        call addmat(ld,temp2,temp3,temp3,n,nm,1)
*
        if (hom .ne. 1) call addmat(ld,f20,temp3,temp3,n,nm,1)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,n,nm,temp3,v(1+n*nm))
*
* Compute derivatives of F3(a,lambda,x)
*
        call setmat(ld,0.d0,temp3,nm,nm)
        if (hom .ne. 1) call addmat(ld,f30,temp3,temp3,nm,nm,1)
*
* Transform result to vector form
*
        xtyp = 2
        call xmat(ld,xtyp,nm,nm,temp3,v(1+2*n*nm))
      endif
*
      return
      end
*
*-----------------------------------------------------------------------
*-----------------------------------------------------------------------
*
*     COMPIJ
*
      subroutine compij(ij,dim1,dim2,i,j)
*
      integer ij,dim1,dim2,i,j,loop
*
      i = 1
      do 301 loop = 1,dim1
        if (ij .le. dim2) then
          j = ij
          return
        endif
        ij = ij - dim2
        i = i + 1
301   continue
*
      return
      end
*
