      subroutine rrho(ld,n,nm,type,hom,lambda,x,a,d,bvbt,ctrc,f10,
     &                f20,f30,v,ww,wi,sigma,temp1,temp2,temp3,aal,mat)
*
* RRHO evaluates rho(a,lambda,x) at given point.
*
* Input parameters:
* ld - leading dimension of matrices
* n,nm - dimensions of system and model
* type - indicates whether RRHO is used for curve tracking (type = 1)
*        or quasi-Newton iterations (type = 2 or 3)
* hom - indicates type of homotopy map: hom = 0 indicates Newton
*       homotopy, hom = 1 indicates rho(a,lambda,x) = F(a,lambda,x)
* lambda,x - input point (lambda,x)
* a,d,bvbt,ctrc - A,D, B V B^t, C^t R C
* f10,f20,f30 - F(0,X(0))
*
* Output parameters:
* v - evaluated rho
*
* Working storage:
* ww,wi,sigma,temp1,temp2,temp3,aal,mat - working matrices
*
      integer hom,ld,n,nm,type
      double precision a(ld,ld),aal(ld,ld),bvbt(ld,ld),ctrc(ld,ld)
      double precision d(ld,ld),f10(ld,ld),f20(ld,ld),f30(ld,ld),lambda
      double precision mat(ld,ld),sigma(ld,ld),t1,temp1(ld,ld)
      double precision temp2(ld,ld),temp3(ld,ld),v(*),wi(ld,ld)
      double precision ww(ld,ld)
*
      call xwus(ld,n,nm,1,ww,wi,sigma,x)
*
* Compute AAL = lambda * A + (1-lambda) * D
*
      t1 = 1.d0 - lambda
      call scmat(ld,lambda,a,temp1,n,n)
      call scmat(ld,t1,d,temp2,n,n)
      call addmat(ld,temp1,temp2,aal,n,n,1)
*
* Determine which matrix is used as A(lambda)
*
      if (type .eq. 1) call scmat(ld,1.d0,aal,mat,n,n)
      if (type .eq. 2) call scmat(ld,1.d0,d,mat,n,n)
      if (type .eq. 3) call scmat(ld,1.d0,a,mat,n,n)
*
* Compute F1(a,lambda,x)
*
      call mulmat(ld,wi,mat,temp1,nm,n,n,1)
      call mulmat(ld,temp1,ww,temp2,nm,nm,n,1)
      call mulmat(ld,temp2,sigma,temp1,nm,nm,nm,1)
      call mulmat(ld,temp1,ww,temp3,nm,n,nm,3)
*
      call mulmat(ld,sigma,ww,temp1,nm,n,nm,3)
      call mulmat(ld,temp1,mat,temp2,nm,n,n,3)
      call addmat(ld,temp2,temp3,temp3,nm,n,1)
*
      call mulmat(ld,wi,bvbt,temp1,nm,n,n,1)
      call addmat(ld,temp1,temp3,temp3,nm,n,1)
*
      if (type .eq. 1 .and. hom .ne. 1) then
        t1 = 1.d0 - lambda
        call scmat(ld,t1,f10,temp1,nm,n)
        call addmat(ld,temp1,temp3,temp3,nm,n,2)
      endif
      call xmat(ld,2,nm,n,temp3,v)
*
* Compute F2(a,lambda,x)
*
      call mulmat(ld,wi,sigma,temp1,n,nm,nm,2)
      call mulmat(ld,temp1,wi,temp2,n,n,nm,1)
      call mulmat(ld,temp2,mat,temp1,n,n,n,1)
      call mulmat(ld,temp1,ww,temp3,n,nm,n,1)
*
      call mulmat(ld,mat,wi,temp1,n,nm,n,4)
      call mulmat(ld,temp1,sigma,temp2,n,nm,nm,1)
      call addmat(ld,temp2,temp3,temp3,n,nm,1)
*
      call mulmat(ld,ctrc,ww,temp1,n,nm,n,1)
      call addmat(ld,temp1,temp3,temp3,n,nm,1)
*
      if (type .eq. 1 .and. hom .ne. 1) then
        call scmat(ld,t1,f20,temp1,n,nm)
        call addmat(ld,temp1,temp3,temp3,n,nm,2)
      endif
      call xmat(ld,2,n,nm,temp3,v(1+n*nm))
*
* Compute F3(a,lambda,x)
*
      call mulmat(ld,wi,ww,temp3,nm,nm,n,1)
      do 351 i = 1,nm
        temp3(i,i) = temp3(i,i) - 1.d0
351   continue
*
      if (type .eq. 1 .and. hom .ne. 1) then
        call scmat(ld,t1,f30,temp1,nm,nm)
        call addmat(ld,temp1,temp3,temp3,nm,nm,2)
      endif
      call xmat(ld,2,nm,nm,temp3,v(1+2*n*nm))
*
      return
      end
