function [cost] = dh2cost(k)
% DH2COST	evaluates the cost function for discrete-time
%		H2-optimal performance.  
%
%		COST = DH2COST(K) evaluates the cost function
%		for discrete-time H2-optimal performance using the 
%		compensator gains contained in the vector K.  
%		DH2COST is a subroutine used by OPTGAIN.
%
%		SEE ALSO: OPTGAIN
%
%

%==================================================================
% 
% REVISED: Richard S. Erwin
%	   February 27, 1996
%	   University of Michigan
% 
%====================================================================
% load the system parameters 
%
    load sourcefile.mat
%
%  initialize variables:
%
    kstart = 1;

    nrk = kindex*indexkr';
    nck = kindex*indexkc';

    K = zeros(nrk,nck);

    for i = 1:N

          nk = indexkr(i)*indexkc(i);

          ki=k(kstart:kstart + nk - 1);
          ki=ivec(ki,indexkr(i),indexkc(i));
              
          for j = 1:kindex(i)
           
                 ijstr = [num2str(i),num2str(j)];
       
		 kstr = ['K = K + QL',ijstr,'*ki*QR',ijstr,';'];
		 eval(kstr)
		
            end;

            kstart = kstart + nk;

     end;

     invLk = inv(eye(nck) - Dyu*K);

     Atilde = A + Bu*K*invLk*Cy;
     Bwtilde = Bw + Bu*K*invLk*Dyw;
     Cztilde = Cz + Dzu*K*invLk*Cy; 

     Dzwtilde = Dzw + Dzu*K*invLk*Dyw;

     if Dzwtilde ~= zeros(size(Dzwtilde))

		error('ERROR: Closed-Loop Feed-Through Matrx Dzwtilde is NONZERO!')

      end

      Vtilde  = Bwtilde*Bwtilde';
      Rtilde  = Cztilde'*Cztilde;
      Ptilde  = dlyap(Atilde',Rtilde);

      cost = trace(Ptilde*Vtilde);

    end;



