function grad = pvgrad(k)

% PVGRAD	computes the gradient of the Lagrangian
%		function for scaled Popov performance.  
%
%		GRAD = PVGRAD(K) computes the gradients of
%		the Lagrangian for scaled Popov performance
%		with respect to the compensator gains 
%		contained in the vector K.  PVGRAD is 
%		a subroutine used by OPTGAIN.
%
%		SEE ALSO: OPTGAIN
%

%==================================================================
%
% REVISED: Richard S. Erwin
%          08 July 1995
%          University of Michigan
%
%==================================================================
% load the system parameters 
%

      load sourcefile.mat
%
%  initialize variables:
%
    kstart = 1;

    nrk = kindex*indexkr';
    nck = kindex*indexkc';

    K = zeros(nrk,nck);

    for i = 1:N

          nk = indexkr(i)*indexkc(i);

          ki=k(kstart:kstart + nk - 1);
          ki=ivec(ki,indexkr(i),indexkc(i));
              
          for j = 1:kindex(i)
           
                 ijstr = [num2str(i),num2str(j)];
       
		 kstr = ['K = K + QL',ijstr,'*ki*QR',ijstr,';'];
		 eval(kstr)
		
            end;

            kstart = kstart + nk;

      end;

      invLk = inv(eye(nck) - Dyu*K);

      Atilde = A + Bu*K*invLk*Cy;
      Bwtilde = Bw + Bu*K*invLk*Dyw;
      Cztilde = Cz + Dzu*K*invLk*Cy; 

      Dzwtilde = Dzw + Dzu*K*invLk*Dyw;

      if Dzwtilde ~= zeros(size(Dzwtilde))

		error('ERROR: Closed-Loop Feed-Through Matrx Dzwtilde is NONZERO!')

      end

      Bdtilde = Bd + Bu*K*invLk*Dyd;
      Cetilde = Ce + Deu*K*invLk*Cy;

      Vtilde  = Bwtilde*Bwtilde';
      Rtilde  = Cztilde'*Cztilde;

      wvec = k(kstart:kstart+scum(nblk)-1);
      zvec = k(kstart+scum(nblk):kstart+2*scum(nblk)-1);

      WW = [];
      ZZ = [];

      for i = 1:nblk,

        if blk(i,1) == 1,

          WW = daug(WW,wvec(scum(i)));
          ZZ = daug(ZZ,zvec(scum(i)));

        elseif blk(i,1) == 2,

          WW = daug(WW,ivec(wvec((scum(i)-3):scum(i)),2,2));
          ZZ = daug(ZZ,ivec(zvec((scum(i)-3):scum(i)),2,2));

        end

      end

      invGam = inv(gam*ZZ - WW*Cetilde*Bdtilde - Bdtilde'*Cetilde'*WW);
      A0tilde   = Atilde - inv(gam)*Bdtilde*Cetilde;
      XXhat  = Cetilde'*ZZ + A0tilde'*Cetilde'*WW;

%
%  form hamiltonian for Riccatti equation:
%
%	A'X + X*A + X*R*X - Q = 0
%
%  hamiltonian is:
%
%	H = [ A     R ]
%           [ Q    -A']
%

      A = A0tilde + Bdtilde*invGam*XXhat';
      R = Bdtilde*invGam*Bdtilde';
      Q = -(Rtilde + XXhat*invGam*XXhat');

      H = [A R; Q -A'];

      [P1,P2,flag2] = ric_schr(H);
      Ptilde = P2/P1;

%
% form matrices for Qtilde Lyapunov equation:
%
%    A*X + X*A' + Q = 0
%

     clear A Q R
     XX = XXhat + Ptilde*Bdtilde;

     A = A0tilde + Bdtilde*invGam*XX';
     Q = Vtilde;

     Qtilde = lyap(A,Q);
%
% form dL/dki, convert to vector form, and append to dL/dk

      term1 = eye(nrk) + Dyu'*invLk'*K';
      term2 = Bu'*Ptilde*Bwtilde*Dyw' + Dzu'*Cztilde*Qtilde*Cy' + Bu'*Ptilde*Qtilde*Cy';
      term2 = term2 + Bu'*Ptilde*Qtilde*XX*invGam*Dyd';
      term2 = term2 + Deu'*ZZ*invGam*XX'*Qtilde*Cy';
      term2 = term2 + Deu'*WW*invGam*XX'*Qtilde*A0tilde'*Cy' + Bu'*Cetilde'*WW*invGam*XX'*Qtilde*Cy';
      term2 = term2 - inv(gam)*Bu'*Cetilde'*WW*invGam*XX'*Qtilde*Cetilde'*Dyd';
      term2 = term2 - inv(gam)*Deu'*Bdtilde'*Cetilde'*WW*invGam*XX'*Qtilde*Cy';
      term2 = term2 - Deu'*WW*invGam*XX'*Qtilde*XX*invGam*Bdtilde'*Cy';
      term2 = term2 - Bu'*Cetilde'*WW*invGam*XX'*Qtilde*XX*invGam*Dyd';
      term2 = term2 + 2*inv(gam)*(Deu'*WW*Cetilde*Vtilde*Cy' + Bu'*Cetilde'*WW*Cetilde*Bwtilde*Dyw');
      term2 = term2 - inv(gam)*(Bu'*Ptilde*Qtilde*Cetilde'*Dyd' + Deu'*Bdtilde'*Ptilde*Qtilde*Cy');

      dLdK = term1*term2*invLk';

      for i = 1 : N

	 dLdki = zeros(indexkr(i),indexkc(i));
               
         for j = 1:kindex(i)
                
            ijstr = [num2str(i),num2str(j)];
   
	    qlstr = ['QLij = QL',ijstr,';'];
 	    qrstr = ['QRij = QR',ijstr,';'];

	    eval(qlstr)
	    eval(qrstr)

            dLdki = dLdki + 2*QLij'*dLdK*QRij';

         end

         vdLdki = vec(dLdki);

         dLdk = [dLdk;vdLdki];

     end;

%
%  compute the gradients of the Lagrangian w/resp to WW and ZZ
%
	dterm1 = inv(gam)*Cetilde*Vtilde*Cetilde';
	dterm2 = invGam*XX'*Qtilde*XX*invGam*Bdtilde'*Cetilde';
	dterm3 = Cetilde*A0tilde*Qtilde*XX*invGam;
	dLdWW  = dterm1 + dterm2 + dterm2' + dterm3 + dterm3';

	clear dterm1 dterm2 dterm3	

	dterm1 = Cetilde*Qtilde*XX*invGam;
	dterm2 = -inv(gam)*invGam*XX'*Qtilde*XX*invGam;
	dLdZZ = dterm1 + dterm1' + dterm2;
	
	clear dterm1 dterm2

%
%  vectorize these gradient matrices
%

     vdLdWW = [];
     vdLdZZ = [];

      for i = 1:nblk,

       if blk(i,1) == 1,

         vdLdWW = [vdLdWW; dLdWW(cum(i),cum(i))];
         vdLdZZ = [vdLdZZ; dLdZZ(cum(i),cum(i))];

       elseif blk(i,1) == 2,

         vdLdWW = [vdLdWW; vec(dLdWW(cum(i)-1:cum(i),cum(i)-1:cum(i)))];
         vdLdZZ = [vdLdZZ; vec(dLdZZ(cum(i)-1:cum(i),cum(i)-1:cum(i)))];

       end,

     end

     grad = [dLdk; vdLdWW; vdLdZZ];
