function flag = stable(k)

%function flag = stable(k)
%
% STABLE   Generates a flag which indicates whether the 
%          closed-loop system is asymptotically stable and 
%          satisfies all constraints.
%  
%          FLAG = STABLE(k) indicates whether the closed loop system 
%          is asymptotically stable and satisfies all constraints with the
%          gain matrices contained within K.  If FLAG = 1, all constraints 
%          were satisfied, while FLAG = 0 implies an attempted
%          constraint violation.
% 

%===========================================================
%
% REVISED: Richard S. Erwin
%          08 July 1995
%          University of Michigan
%
%===========================================================
%
%   disp(k)
%

% load the system parameters 
%

    load sourcefile.mat

% initialize variables:
%
    flag = 1;
    flag2 = 0;
    flag1 = 0;

    kstart = 1;

    nrk = kindex*indexkr';
    nck = kindex*indexkc';

    K = zeros(nrk,nck);

    for i = 1:N

          nk = indexkr(i)*indexkc(i);

          ki=k(kstart:kstart + nk - 1);
          ki=ivec(ki,indexkr(i),indexkc(i));
              
          for j = 1:kindex(i)
           
                 ijstr = [num2str(i),num2str(j)];
       
		 kstr = ['K = K + QL',ijstr,'*ki*QR',ijstr,';'];
		 eval(kstr)
		
            end;

            kstart = kstart + nk;

      end;

      invLk = inv(eye(nck) - Dyu*K);


      Atilde = A + Bu*K*invLk*Cy;

%
%  all  controllers:  Check closed loop stability
%  flag1 = 1 --> closed loop system is unstable
%

    z = eig(Atilde);


if ctype > 0

    	realroot = real(z);
    	flag1 = max(realroot >= 0);

else 

    	testvec = abs(z);
    	flag1 = max(testvec >= 1); 

end


if flag1 == 1

	pstr = ['ATTEMPTED STABILITY CONSTRAINT VIOLATION'];
	fprintf(1,'%c',pstr);
	fprintf(1,'%c \n',' ');
	flag = 0;

else


%
% Performance Constraints
%

	if abs(ctype) ~= 1

      		Bwtilde = Bw + Bu*K*invLk*Dyw;
      		Cztilde = Cz + Dzu*K*invLk*Cy; 

      		Dzwtilde = Dzw + Dzu*K*invLk*Dyw;

      		if Dzwtilde ~= zeros(size(Dzwtilde))

			error('ERROR: Closed-Loop Feed-Through Matrx Dzwtilde is NONZERO!')

      		end

      		Vtilde  = Bwtilde*Bwtilde';
      		Rtilde  = Cztilde'*Cztilde;

      		Bdtilde = Bd + Bu*K*invLk*Dyd;
      		Cetilde = Ce + Deu*K*invLk*Cy;

   		if ctype == 3

			flag2 = pv_const(k,blk,nblk,kstart,gam,Atilde,Bwtilde,Cztilde,Bdtilde,Cetilde);

   		end
   	end


     	if flag2 ~= 0,

  		flag = 0;

     	end

end
